@@ -3,6 +3,7 @@ import queue
 import threading
 import time
 from queue import Queue
+from typing import Collection
 from typing import Optional
 
 from kubernetes import watch
@@ -45,12 +46,21 @@ SUPPORTED_POD_MODIFIED_EVENT_PHASES = {
 class KubernetesPodExecutor(TaskExecutor):
     TASK_CONFIG_INTERFACE = KubernetesTaskConfig
 
-    def __init__(self, namespace: str, kubeconfig_path: Optional[str] = None) -> None:
+    def __init__(
+        self,
+        namespace: str,
+        kubeconfig_path: Optional[str] = None,
+        task_configs: Optional[Collection[KubernetesTaskConfig]] = [],
+    ) -> None:
         self.kube_client = KubeClient(kubeconfig_path=kubeconfig_path)
         self.namespace = namespace
         self.stopping = False
         self.task_metadata: PMap[str, KubernetesTaskMetadata] = pmap()
+
         self.task_metadata_lock = threading.RLock()
+        if task_configs:
+            for task_config in task_configs:
+                self._initialize_existing_task(task_config)
 
         # we have two queues since we need to process Pod Events into something that we can place
         # onto the queue that any application (e.g., something like Tron or Jolt) can actually use
@@ -79,6 +89,28 @@ class KubernetesPodExecutor(TaskExecutor):
         )
         self.pending_event_processing_thread.start()
 
+    def _initialize_existing_task(self, task_config: KubernetesTaskConfig) -> None:
+        """ Generates task_metadata in UNKNOWN state for an existing KubernetesTaskConfig.
+            Used during initialization or recovery for a task"""
+        pod_name = task_config.pod_name
+        logger.debug(
+            f"Initializing task metadata for known pod {pod_name}"
+        )
+        # We are initiating with UNKNOWN state on initial load, then leave
+        # matching tasks to running/completed pods in reconcile()
+        with self.task_metadata_lock:
+            self.task_metadata = self.task_metadata.set(
+                pod_name,
+                KubernetesTaskMetadata(
+                    task_config=task_config,
+                    node_name='UNKNOWN',
+                    task_state=KubernetesTaskState.TASK_UNKNOWN,
+                    task_state_history=v(
+                        (KubernetesTaskState.TASK_UNKNOWN, time.time())
+                    ),
+                )
+            )
+
     def _pod_event_watch_loop(self) -> None:
         logger.debug(f"Starting watching Pod events for namespace={self.namespace}.")
         # TODO(TASKPROC-243): we'll need to correctly handle resourceVersion expiration for the case
@@ -123,6 +155,7 @@ class KubernetesPodExecutor(TaskExecutor):
         pod = event["object"]
         pod_name = pod.metadata.name
         task_metadata = self.task_metadata[pod_name]
+        raw_event = event['raw_object']
 
         logger.info(f"Removing {pod_name} from state and emitting 'killed' event.")
 
@@ -133,7 +166,7 @@ class KubernetesPodExecutor(TaskExecutor):
                 terminal=True,
                 success=False,
                 timestamp=time.time(),
-                raw=event["raw_object"],
+                raw=raw_event,
                 task_config=task_metadata.task_config,
                 platform_type="killed"
             )
@@ -141,9 +174,15 @@ class KubernetesPodExecutor(TaskExecutor):
 
     def __handle_modified_pod_event(self, event: PodEvent) -> None:
         pod = event["object"]
+        self.__update_modified_pod(pod=pod, event=event)
+
+    def __update_modified_pod(self, pod: V1Pod, event: Optional[PodEvent]) -> None:
+        """ Called during reconciliation and normal event handling """
         pod_name = pod.metadata.name
         task_metadata = self.task_metadata[pod_name]
 
+        raw_event = event['raw_object'] if event else None
+
         if pod.status.phase not in SUPPORTED_POD_MODIFIED_EVENT_PHASES:
             logger.debug(
                 f"Got a MODIFIED event for {pod_name} for unhandled phase: "
@@ -165,7 +204,7 @@ class KubernetesPodExecutor(TaskExecutor):
                     terminal=True,
                     success=True,
                     timestamp=time.time(),
-                    raw=event["raw_object"],
+                    raw=raw_event,
                     task_config=task_metadata.task_config,
                     platform_type="finished"
                 )
@@ -184,7 +223,7 @@ class KubernetesPodExecutor(TaskExecutor):
                     terminal=True,
                     success=False,
                     timestamp=time.time(),
-                    raw=event["raw_object"],
+                    raw=raw_event,
                     task_config=task_metadata.task_config,
                     platform_type="failed"
                 )
@@ -199,7 +238,7 @@ class KubernetesPodExecutor(TaskExecutor):
             self.task_metadata = self.task_metadata.set(
                 pod_name,
                 task_metadata.set(
-                    node_name=pod.status.host_ip,
+                    node_name=pod.spec.node_name,
                     task_state=KubernetesTaskState.TASK_RUNNING,
                     task_state_history=task_metadata.task_state_history.append(
                         (KubernetesTaskState.TASK_RUNNING, time.time()),
@@ -211,7 +250,7 @@ class KubernetesPodExecutor(TaskExecutor):
                     task_id=pod_name,
                     terminal=False,
                     timestamp=time.time(),
-                    raw=event["raw_object"],
+                    raw=raw_event,
                     task_config=task_metadata.task_config,
                     platform_type="running"
                 )
@@ -244,7 +283,7 @@ class KubernetesPodExecutor(TaskExecutor):
                     task_id=pod_name,
                     terminal=False,
                     timestamp=time.time(),
-                    raw=event["raw_object"],
+                    raw=raw_event,
                     task_config=task_metadata.task_config,
                     platform_type="lost"
                 )
@@ -391,10 +430,53 @@ class KubernetesPodExecutor(TaskExecutor):
         return None
 
     def reconcile(self, task_config: KubernetesTaskConfig) -> None:
-        # TASKPROC(244): we probably only want reconcile() to fill in the task_config
-        # member of the relevant KubernetesTaskMetadata and do an unconditional state
-        # reconciliation on startup
-        pass
+        pod_name = task_config.pod_name
+        try:
+            pod = self.kube_client.get_pod(namespace=self.namespace, pod_name=pod_name)
+        except Exception:
+            logger.exception(f"Hit an exception attempting to fetch pod {pod_name}")
+            pod = None
+
+        if pod_name not in self.task_metadata:
+            self._initialize_existing_task(task_config)
+
+        with self.task_metadata_lock:
+            task_metadata = self.task_metadata[pod_name]
+            self.task_metadata = self.task_metadata.set(
+                pod_name,
+                task_metadata.set(
+                    task_config=task_config
+                )
+            )
+
+            if not pod:
+                # Pod has gone away while restarting
+                logger.info(
+                    f"Pod {pod_name} for task {task_config.name} was no longer found. "
+                    "Marking as LOST"
+                )
+                self.task_metadata = self.task_metadata.set(
+                    pod_name,
+                    task_metadata.set(
+                        task_state=KubernetesTaskState.TASK_LOST,
+                        task_state_history=task_metadata.task_state_history.append(
+                            (KubernetesTaskState.TASK_LOST, time.time()),
+                        )
+                    )
+                )
+                self.event_queue.put(
+                    task_event(
+                        task_id=pod_name,
+                        terminal=False,
+                        timestamp=time.time(),
+                        raw=None,
+                        task_config=task_metadata.task_config,
+                        platform_type="lost"
+                    )
+                )
+            else:
+                # Treat like a modified pod
+                self.__update_modified_pod(pod=pod, event=None)
 
     def kill(self, task_id: str) -> bool:
         """
